@@ -229,7 +229,8 @@ class DatasetMaker:
         return self(name.replace("_", "-"))
 
 
-dataset = DatasetMaker()
+get_dataset = DatasetMaker()
+
 
 TERMS_OF_USE_SHOWN = set()
 
@@ -247,7 +248,7 @@ def load_dataset(name: str, *args, **kwargs) -> Dataset:
     Dataset
         The loaded dataset.
     """
-    klass = dataset.lookup(name)
+    klass = get_dataset.lookup(name)
 
     if name not in TERMS_OF_USE_SHOWN:
         if klass.terms_of_use is not None:
