@@ -219,14 +219,14 @@ class GribField(Base):
     def shape(self):
         return self.handle.get("Nj"), self.handle.get("Ni")
 
-    def plot_map(self, driver):
-        driver.bounding_box(
+    def plot_map(self, backend):
+        backend.bounding_box(
             north=self.handle.get("latitudeOfFirstGridPointInDegrees"),
             south=self.handle.get("latitudeOfLastGridPointInDegrees"),
             west=self.handle.get("longitudeOfFirstGridPointInDegrees"),
             east=self.handle.get("longitudeOfLastGridPointInDegrees"),
         )
-        driver.plot_grib(self.path, self.handle.get("offset"))
+        backend.plot_grib(self.path, self.handle.get("offset"))
 
     @call_counter
     def to_numpy(self):
@@ -504,7 +504,47 @@ class GRIBReader(Reader):
             **xarray_open_mfdataset_kwargs,
         )
 
+    def to_metview(self):
+        from climetlab.metview import mv_read
 
-def reader(source, path, magic, deeper_check):
-    if magic[:4] == b"GRIB":
+        return mv_read(self.path)
+
+    def plot_map(self, backend):
+        return self.first.plot_map(backend)
+
+    def plot_graph(self, backend):
+        import numpy as np
+
+        what = backend._options("what", "global_average")
+        what = dict(
+            global_average=np.mean,
+        )[what]
+
+        # initialize list of lists
+        data = [[s.valid_datetime(), what(s.to_numpy())] for s in self]
+        import pandas as pd
+
+        df = pd.DataFrame(data, columns=["date", "param"])
+
+        backend.plot_graph_add_timeserie(df)
+
+    # Used by normalisers
+    def to_datetime(self):
+        times = self.to_datetime_list()
+        assert len(times) == 1
+        return times[0]
+
+    def to_datetime_list(self):
+        # TODO: check if that can be done faster
+        result = set()
+        for s in self:
+            result.add(s.valid_datetime())
+        return sorted(result)
+
+    def to_bounding_box(self):
+        return BoundingBox.multi_merge([s.to_bounding_box() for s in self])
+
+
+def reader(source, path, magic=None, deeper_check=False):
+    if magic is None or magic[:4] == b"GRIB":
         return GRIBReader(source, path)
