@@ -81,6 +81,7 @@ def make_xarray(
     dims=["lat", "lon"],
     size=2,
     coord_values=None,
+    attributes=None,
     **kwargs,
 ):
     import numpy as np
@@ -116,6 +117,18 @@ def make_xarray(
 
     ds = xr.Dataset(vars)
 
+    if "lat" in ds.dims:
+        ds["lat"].attrs["standard_name"] = "latitude"
+    if "lon" in ds.dims:
+        ds["lon"].attrs["standard_name"] = "longitude"
+    if "time" in ds.dims:
+        ds["time"].attrs["standard_name"] = "time"
+
+    if attributes:
+        for d, attrs in attributes.items():
+            for k, v in attrs.items():
+                ds[d].attrs[k] = v
+
     return ds
 
 
