@@ -21,7 +21,15 @@ from . import Source
 LOG = logging.getLogger(__name__)
 
 
-class FileSource(Source, os.PathLike):
+class FileSourceMeta(type(Source), type(os.PathLike)):
+    def patch(cls, obj, *args, **kwargs):
+        if "reader" in kwargs:
+            setattr(obj, "reader", kwargs.pop("reader"))
+
+        return super().patch(obj, *args, **kwargs)
+
+
+class FileSource(Source, os.PathLike, metaclass=FileSourceMeta):
 
     _reader_ = None
     path = None
@@ -70,6 +78,9 @@ class FileSource(Source, os.PathLike):
     def sel(self, **kwargs):
         return self._reader.sel(**kwargs)
 
+    def plot_graph(self, *args, **kwargs):
+        return self._reader.plot_graph(*args, **kwargs)
+
     def plot_map(self, *args, **kwargs):
         return self._reader.plot_map(*args, **kwargs)
 
@@ -100,13 +111,30 @@ class FileSource(Source, os.PathLike):
 
     def __repr__(self):
         cache_dir = SETTINGS.get("cache-directory")
+        path = self.path
         if isinstance(self.path, str):
-            path = self.path.replace(cache_dir, "CACHE:")
-        return f"{self.__class__.__name__}({path},{self._reader.__class__.__name__})"
+            path = path.replace(cache_dir, "CACHE:")
+        try:
+            reader_class_name = str(self._reader.__class__.name)
+        except AttributeError:  # TODO: improve this
+            reader_class_name = "unknown"
+        except:  # TODO: improve this # noqa: E722
+            reader_class_name = "Unknown"
+        return f"{self.__class__.__name__}({path},{reader_class_name})"
 
     def __fspath__(self):
         return self.path
 
+    # Used by normalisers
+    def to_datetime(self):
+        return self._reader.to_datetime()
+
+    def to_datetime_list(self):
+        return self._reader.to_datetime_list()
+
+    def to_bounding_box(self):
+        return self._reader.to_bounding_box()
+
 
 class File(FileSource):
     def __init__(
