@@ -13,6 +13,7 @@ import json
 import logging
 import mimetypes
 import os
+import re
 import sys
 from ftplib import FTP
 from urllib.parse import urlparse
@@ -29,17 +30,6 @@ from .file import FileSource
 
 LOG = logging.getLogger(__name__)
 
-OFFLINE = False  # For use with pytest
-
-
-def offline(off):
-    global OFFLINE
-    OFFLINE = off
-
-
-def dummy_watcher():
-    pass
-
 
 def mimetype_to_extension(mimetype, compression, default=".unknown"):
     EXTENSIONS = {
@@ -71,6 +61,9 @@ class Downloader:
         # TODO: use weakref instead
         self.owner = owner
 
+        if self.owner.parts:
+            assert self.supports_parts
+
     def local_path(self, url):
         return None
 
@@ -108,7 +101,7 @@ class Downloader:
         ) as pbar:
 
             with open(download, mode) as f:
-                total = self.transfer(f, pbar, self.owner.watcher)
+                total = self.transfer(f, pbar, self.owner.observer)
 
             pbar.close()
 
@@ -135,7 +128,105 @@ class Downloader:
         return False
 
 
+class DecodeMultipart:
+    def __init__(self, request, parts):
+        self.request = request
+        assert request.status_code == 206, request.status_code
+
+        kind, self.boundary = request.headers["content-type"].split("=")
+        assert kind == "multipart/byteranges; boundary", kind
+
+        self.size = int(request.headers["content-length"])
+        self.encoding = "utf-8"
+        self.parts = parts
+
+    def __call__(self, chunk_size):
+        return self.stream(chunk_size)
+
+    def stream(self, chunk_size):
+        from email.parser import HeaderParser
+
+        from requests.structures import CaseInsensitiveDict
+
+        header_parser = HeaderParser()
+        marker = f"--{self.boundary}\r\n".encode(self.encoding)
+        end_header = b"\r\n\r\n"
+        end_data = b"\r\n"
+
+        end_of_input = f"--{self.boundary}--\r\n".encode(self.encoding)
+
+        if chunk_size < len(end_data):
+            chunk_size = len(end_data)
+
+        iter_content = self.request.iter_content(chunk_size)
+        chunk = next(iter_content)
+
+        # Some servers start with \r\n
+        if chunk[:2] == end_data:
+            chunk = chunk[2:]
+
+        LOG.debug("MARKER %s", marker)
+        part = 0
+        while True:
+            while len(chunk) < max(len(marker), len(end_of_input)):
+                more = next(iter_content)
+                assert more is not None
+                chunk += more
+
+            if chunk.find(end_of_input) == 0:
+                assert part == len(self.parts)
+                break
+
+            pos = chunk.find(marker)
+            assert pos == 0, (pos, chunk)
+
+            chunk = chunk[pos + len(marker) :]
+            while True:
+                pos = chunk.find(end_header)
+                if pos != -1:
+                    break
+                more = next(iter_content)
+                assert more is not None
+                chunk += more
+                assert len(chunk) < 1024 * 16
+
+            pos += len(end_header)
+            header = chunk[:pos].decode(self.encoding)
+            header = CaseInsensitiveDict(header_parser.parsestr(header))
+            chunk = chunk[pos:]
+            # kind = header["content-type"]
+            bytes = header["content-range"]
+            LOG.debug("HEADERS %s", header)
+            m = re.match(r"^bytes (\d+)d?-(\d+)d?/(\d+)d?$", bytes)
+            assert m, header
+            start, end, total = int(m.group(1)), int(m.group(2)), int(m.group(3))
+
+            assert end >= start
+            assert start < total
+            assert end < total
+
+            size = end - start + 1
+
+            assert start == self.parts[part][0]
+            assert end == self.parts[part][0] + self.parts[part][1] - 1
+
+            while size > 0:
+                if len(chunk) >= size:
+                    yield chunk[:size]
+                    chunk = chunk[size:]
+                    size = 0
+                else:
+                    yield chunk
+                    size -= len(chunk)
+                    chunk = next(iter_content)
+
+            assert chunk.find(end_data) == 0
+            chunk = chunk[len(end_data) :]
+            part += 1
+
+
 class HTTPDownloader(Downloader):
+    supports_parts = True
 
     _headers = None
     _url = None
@@ -162,20 +253,27 @@ class HTTPDownloader(Downloader):
                         json.dumps(self._headers, sort_keys=True, indent=4),
                     )
                 except Exception:
+                    self._url = None
+                    self._headers = {}
                     LOG.exception("HEAD %s", url)
         return self._headers
 
     def extension(self, url):
 
-        headers = self.headers(url)
-
         ext = super().extension(url)
 
-        if "content-disposition" in headers:
-            value, params = cgi.parse_header(headers["content-disposition"])
-            assert value == "attachment", value
-            if "filename" in params:
-                ext = super().extension(params["filename"])
+        if ext == ".unknown":
+            # Only check for "content-disposition" if
+            # the URL does not end with an extension
+            # so we avoid fetching the headers unesseraly
+
+            headers = self.headers(url)
+
+            if "content-disposition" in headers:
+                value, params = cgi.parse_header(headers["content-disposition"])
+                assert value == "attachment", value
+                if "filename" in params:
+                    ext = super().extension(params["filename"])
 
         return ext
 
@@ -194,6 +292,8 @@ class HTTPDownloader(Downloader):
         mode = "wb"
         skip = 0
 
+        parts = self.owner.parts
+
         headers = self.headers(url)
         if "content-length" in headers:
             try:
@@ -206,7 +306,8 @@ class HTTPDownloader(Downloader):
         encoded = headers.get("content-encoding") is not None
 
         http_headers = dict(**self.owner.http_headers)
-        if not encoded and os.path.exists(download):
+
+        if not parts and not encoded and os.path.exists(download):
 
             bytes = os.path.getsize(download)
 
@@ -230,6 +331,17 @@ class HTTPDownloader(Downloader):
                         bytes,
                     )
 
+        if parts:
+            if headers.get("accept-ranges") != "bytes":
+                raise ValueError(f"Server for {url} does not support byte ranges")
+            # We can trust the size
+            encoded = None
+            size = sum(p[1] for p in parts)
+            ranges = []
+            for offset, length in parts:
+                ranges.append(f"{offset}-{offset+length-1}")
+            http_headers["range"] = f"bytes={','.join(ranges)}"
+
         r = requests.get(
             url,
             stream=True,
@@ -239,13 +351,22 @@ class HTTPDownloader(Downloader):
         )
         r.raise_for_status()
 
-        self.request = r
+        self.stream = r.iter_content
+        if parts and len(parts) > 1:
+            self.stream = DecodeMultipart(r, parts)
 
+        LOG.debug(
+            "url prepare size=%s mode=%s skip=%s encoded=%s",
+            size,
+            mode,
+            skip,
+            encoded,
+        )
         return size, mode, skip, encoded
 
     def transfer(self, f, pbar, watcher):
         total = 0
-        for chunk in self.request.iter_content(chunk_size=1024 * 1024):
+        for chunk in self.stream(chunk_size=1024 * 1024):
             watcher()
             if chunk:
                 f.write(chunk)
@@ -281,11 +402,15 @@ class HTTPDownloader(Downloader):
                             cache_data["expires"],
                         )
 
-            headers = self.headers(url)
+            try:
+                headers = self.headers(url)
+            except requests.exceptions.ConnectionError:
+                return False
+
             cached_etag = cache_data.get("etag")
             remote_etag = headers.get("etag")
 
-            if cached_etag != remote_etag:
+            if cached_etag != remote_etag and remote_etag is not None:
                 LOG.warning("Remote content of URL %s has changed", url)
                 if (
                     SETTINGS.get("download-out-of-date-urls")
@@ -304,6 +429,9 @@ class HTTPDownloader(Downloader):
 
 
 class FTPDownloader(Downloader):
+
+    supports_parts = False
+
     def prepare(self, url, download):
 
         mode = "wb"
@@ -352,7 +480,10 @@ class FTPDownloader(Downloader):
 
 
 class FileDownloader(Downloader):
+    supports_parts = True
+
     def local_path(self, url):
+
         o = urlparse(url)
         path = o.path
 
@@ -365,7 +496,33 @@ class FileDownloader(Downloader):
         if sys.platform == "win32" and path[0] == "/" and path[2] == ":":
             path = path[1:]
 
-        return path
+        self.path = path
+
+        # If parts is given, we cannot use the original path
+        return path if self.owner.parts is None else None
+
+    def prepare(self, url, download):
+        parts = self.owner.parts
+        size = sum(p[1] for p in parts)
+        mode = "wb"
+        skip = 0
+        encoded = None
+        return size, mode, skip, encoded
+
+    def transfer(self, f, pbar, watcher):
+        with open(self.path, "rb") as g:
+            total = 0
+            for offset, length in self.owner.parts:
+                g.seek(offset)
+                watcher()
+                while length > 0:
+                    chunk = g.read(min(length, 1024 * 1024))
+                    assert chunk
+                    f.write(chunk)
+                    length -= len(chunk)
+                    total += len(chunk)
+                    pbar.update(len(chunk))
+        return total
 
 
 DOWNLOADERS = dict(
@@ -380,17 +537,18 @@ class Url(FileSource):
     def __init__(
         self,
         url,
+        parts=None,
+        offsets=None,
+        lengths=None,
         filter=None,
         merger=None,
         verify=True,
-        watcher=None,
         force=None,
         # extension=None,
         http_headers=None,
         update_if_out_of_date=False,
         mirror=DEFAULT_MIRROR,
         fake_headers=None,  # When HEAD is not allowed but you know the size
-        **kwargs,
     ):
         # TODO: re-enable this feature
         extension = None
@@ -401,11 +559,38 @@ class Url(FileSource):
         self.filter = filter
         self.merger = merger
         self.verify = verify
-        self.watcher = watcher if watcher else dummy_watcher
         self.update_if_out_of_date = update_if_out_of_date
         self.http_headers = http_headers if http_headers else {}
         self.fake_headers = fake_headers
 
+        if parts is not None:
+            assert offsets is None and lengths is None
+            offsets = [p[0] for p in parts]
+            lengths = [p[1] for p in parts]
+
+        self.parts = None
+        if offsets is not None or lengths is not None:
+            assert len(offsets) == len(lengths)
+            self.parts = []
+            last = -1
+            # Compress and check
+            for offset, length in zip(offsets, lengths):
+                assert offset >= 0 and length > 0
+                assert offset >= last, (
+                    f"Offsets and lengths must be in order, and not overlapping:"
+                    f" offset={offset}, end of previous part={last}"
+                )
+                if offset == last:
+                    # Compress
+                    offset, prev_length = self.parts.pop()
+                    length += prev_length
+
+                self.parts.append((offset, length))
+                last = offset + length
+
+            if len(self.parts) == 0:
+                self.parts = None
+
         if mirror:
             url = mirror(url)
 
@@ -426,7 +611,6 @@ class Url(FileSource):
             force = downloader.out_of_date
 
         def download(target, url):
-            assert not OFFLINE
             downloader.download(url, target)
             return downloader.cache_data(url)
 
@@ -435,6 +619,7 @@ class Url(FileSource):
             url,
             extension=extension,
             force=force,
+            hash_extra=self.parts,
         )
 
     def __repr__(self) -> str:
