@@ -65,9 +65,10 @@ class ZarrS3(Source):
         # self._ds = xr.concat(dslist, dim = 'head_time')
         # return self._ds
 
-        options = self.read_zarr_options()
+        # options = self.read_zarr_options()
 
-        concat_dim = options.get("concat_dim", "forecast_time")
+        # concat_dim = options.get("concat_dim", "forecast_time")
+        concat_dim = "forecast_time"  # TODO: fix me
 
         stores = [url_to_store(url) for url in urls]
 
