@@ -6,42 +6,49 @@
 # granted to it by virtue of its status as an intergovernmental organisation
 # nor does it submit to any jurisdiction.
 #
+
 from climetlab.wrappers import get_wrapper
 
 
+def _normalize(lon, minimum):
+    while lon < minimum:
+        lon += 360
+
+    while lon >= minimum + 360:
+        lon -= 360
+
+    return lon
+
+
 class BoundingBox:
     def __init__(self, *, north, west, south, east):
+
         # Convert to float as these values may come from Numpy
         self.north = min(float(north), 90.0)
-        self.west = float(west)
         self.south = max(float(south), -90.0)
-        self.east = float(east)
 
-        if self.north <= self.south:
+        self.is_periodic_west_east = (east - west) == 360
+        self.west = _normalize(float(west), -180)  # Or 0?
+
+        if self.is_periodic_west_east:
+            self.east = self.west + 360
+        else:
+            self.east = _normalize(float(east), self.west)
+
+        if self.north < self.south:
             raise ValueError(
-                "North (%s) must be greater than south (%s)"
-                % (
-                    self.north,
-                    self.south,
-                )
+                f"Invalid bounding box, north={self.north} < south={self.south}"
             )
 
-        if self.west == self.east:
-            raise ValueError("West (%s) is equal to east (%s)" % (west, east))
+        if self.west > self.east:
+            raise ValueError(
+                f"Invalid bounding box, west={self.west} > east={self.east}"
+            )
 
-        while self.east < self.west:
-            self.east += 360
-
-        while self.east - self.west > 360:
-            self.east -= 360
-
-        while self.east >= 360 and self.west >= 360:
-            self.east -= 360
-            self.west -= 360
-
-        while self.east < -180 and self.west < -180:
-            self.east += 360
-            self.west += 360
+        if self.east > self.west + 360:
+            raise ValueError(
+                f"Invalid bounding box, east={self.east} > west={self.west}+360"
+            )
 
     def __repr__(self):
         return "BoundingBox(north=%g,west=%g,south=%g,east=%g)" % (
@@ -54,6 +61,7 @@ class BoundingBox:
     def __eq__(self, other):
         if self.__class__ is not other.__class__:
             return False
+
         return self.as_tuple() == other.as_tuple()
 
     @property
@@ -64,32 +72,82 @@ class BoundingBox:
     def height(self):
         return self.north - self.south
 
-    def merge(self, other):
+    @classmethod
+    def multi_merge(cls, bboxes):
 
-        west1, east1 = self.west, self.east
-        west2, east2 = other.west, other.east
+        north = max(z.north for z in bboxes)
+        south = min(z.south for z in bboxes)
 
-        while west1 < 0 or east1 < 0:
-            west1 += 360
-            east1 += 360
+        first = bboxes[0]
 
-        while west2 < 0 or east2 < 0:
-            west2 += 360
-            east2 += 360
-
-        if abs(west1 - (west2 + 360)) < abs(west1 - west2):
-            east2, west2 = east2 + 360, west2 + 360
-        elif abs(west2 - (west1 + 360)) < abs(west2 - west1):
-            east1, west1 = east1 + 360, west1 + 360
-
-        # print(west1, east1, west2, east2)
-        return BoundingBox(
-            north=max(self.north, other.north),
-            west=min(west1, west2),
-            south=min(self.south, other.south),
-            east=max(east1, east2),
+        origin = first.east % 360
+        full = BoundingBox(
+            north=north,
+            west=first.west,
+            south=south,
+            east=first.west + 360,
         )
 
+        # Build the list of provided [west, east] intervals
+
+        boundaries = list()
+        stacked_intervals = set()  # To keep track of overlapping intervals
+        for i, b in enumerate(bboxes):
+
+            if b.east - b.west == 360:
+                return full
+
+            west = (b.west - origin) % 360
+            east = (b.east - origin) % 360
+
+            if west > east:
+                stacked_intervals.add(i)
+
+            boundaries.append((west, True, i))
+            boundaries.append((east, False, i))
+
+        boundaries = sorted(boundaries)
+
+        start = 0
+        best = -1
+        west = 0
+        east = 0
+
+        # Find the longest interval *outside* the provided [west, east] intervals
+        # It's complement will be the result
+
+        for cursor, entering, interval in boundaries:
+
+            if entering:
+                if not stacked_intervals:
+                    distance = cursor - start
+                    if distance > best:
+                        best = distance
+                        west = cursor
+                        east = start
+
+                    start = None
+
+                stacked_intervals.add(interval)
+
+            else:  # exiting
+                stacked_intervals.remove(interval)
+                if not stacked_intervals:
+                    start = cursor
+
+        if best <= 0:
+            return full
+
+        return BoundingBox(
+            north=north,
+            west=origin + west,
+            south=south,
+            east=origin + east,
+        )
+
+    def merge(self, other):
+        return self.multi_merge([self, other])
+
     def add_margins(self, margins):
 
         if isinstance(margins, str) and margins[-1] == "%":
@@ -130,6 +188,7 @@ def to_bounding_box(obj):
     if isinstance(obj, (list, tuple)):
         return BoundingBox(north=obj[0], west=obj[1], south=obj[2], east=obj[3])
 
+    print(obj)
     obj = get_wrapper(obj)
 
     return to_bounding_box(obj.to_bounding_box())
